@@ -29,6 +29,7 @@ If this driver is still needed, set up the cloud configuration at
       provider: aws
 
 '''
+from __future__ import absolute_import
 # pylint: disable=E0102
 
 # Import python libs
@@ -42,19 +43,44 @@ import logging
 import salt.utils.cloud
 import salt.config as config
 from salt.utils import namespaced_function
-from salt.cloud.libcloudfuncs import *   # pylint: disable=W0614,W0401
-from salt.cloud.libcloudfuncs import destroy as libcloudfuncs_destroy
-from salt.cloud.exceptions import (
+
+from salt.exceptions import (
     SaltCloudException,
     SaltCloudSystemExit,
     SaltCloudConfigError,
     SaltCloudExecutionTimeout,
     SaltCloudExecutionFailure
 )
+import salt.ext.six as six
+
+try:
+    from salt.cloud.libcloudfuncs import *   # pylint: disable=W0614,W0401
+    from salt.cloud.libcloudfuncs import destroy as libcloudfuncs_destroy
+    from libcloud.compute.types import Provider
+
+    EC2_LOCATIONS = {
+        'ap-northeast-1': Provider.EC2_AP_NORTHEAST,
+        'ap-southeast-1': Provider.EC2_AP_SOUTHEAST,
+        'eu-west-1': Provider.EC2_EU_WEST,
+        'sa-east-1': Provider.EC2_SA_EAST,
+        'us-east-1': Provider.EC2_US_EAST,
+        'us-west-1': Provider.EC2_US_WEST,
+        'us-west-2': Provider.EC2_US_WEST_OREGON
+    }
+    DEFAULT_LOCATION = 'us-east-1'
+    if hasattr(Provider, 'EC2_AP_SOUTHEAST2'):
+        EC2_LOCATIONS['ap-southeast-2'] = Provider.EC2_AP_SOUTHEAST2
+
+    HAS_LIBCLOUD = True
+except ImportError:
+    HAS_LIBCLOUD = False
 
 # Get logging started
 log = logging.getLogger(__name__)
 
+# namespace libcloudfuncs
+get_salt_interface = namespaced_function(get_salt_interface, globals())
+
 # Define the module's virtual name
 __virtualname__ = 'aws'
 
@@ -64,6 +90,9 @@ def __virtual__():
     '''
     Set up the libcloud funcstions and check for AWS configs
     '''
+    if not HAS_LIBCLOUD:
+        return False
+
     try:
         import botocore
         # Since we have botocore, we won't load the libcloud AWS module
@@ -74,7 +103,7 @@ def __virtual__():
     if get_configured_provider() is False:
         return False
 
-    for provider, details in __opts__['providers'].iteritems():
+    for provider, details in six.iteritems(__opts__['providers']):
         if 'provider' not in details or details['provider'] != 'aws':
             continue
 
@@ -126,21 +155,6 @@ def __virtual__():
     return __virtualname__
 
 
-EC2_LOCATIONS = {
-    'ap-northeast-1': Provider.EC2_AP_NORTHEAST,
-    'ap-southeast-1': Provider.EC2_AP_SOUTHEAST,
-    'eu-west-1': Provider.EC2_EU_WEST,
-    'sa-east-1': Provider.EC2_SA_EAST,
-    'us-east-1': Provider.EC2_US_EAST,
-    'us-west-1': Provider.EC2_US_WEST,
-    'us-west-2': Provider.EC2_US_WEST_OREGON
-}
-DEFAULT_LOCATION = 'us-east-1'
-
-if hasattr(Provider, 'EC2_AP_SOUTHEAST2'):
-    EC2_LOCATIONS['ap-southeast-2'] = Provider.EC2_AP_SOUTHEAST2
-
-
 def get_configured_provider():
     '''
     Return the first configured instance.
@@ -229,7 +243,7 @@ def ssh_username(vm_):
         usernames = [usernames]
 
     # get rid of None's or empty names
-    usernames = filter(lambda x: x, usernames)
+    usernames = [x for x in usernames if x]
     # Keep a copy of the usernames the user might have provided
     initial = usernames[:]
 
@@ -239,7 +253,7 @@ def ssh_username(vm_):
             usernames.append(name)
     # Add the user provided usernames to the end of the list since enough time
     # might need to pass before the remote service is available for logins and
-    # the proper username might have passed it's iteration.
+    # the proper username might have passed its iteration.
     # This has detected in a CentOS 5.7 EC2 image
     usernames.extend(initial)
     return usernames
@@ -360,7 +374,7 @@ def create(vm_):
                 vm_['name'], exc
             ),
             # Show the traceback if the debug logging level is enabled
-            exc_info=log.isEnabledFor(logging.DEBUG)
+            exc_info_on_loglevel=logging.DEBUG
         )
         return False
 
@@ -395,7 +409,7 @@ def create(vm_):
         except SaltCloudSystemExit:
             pass
         finally:
-            raise SaltCloudSystemExit(exc.message)
+            raise SaltCloudSystemExit(str(exc))
 
     if tags:
         set_tags(vm_['name'], tags, call='action')
@@ -407,10 +421,20 @@ def create(vm_):
         log.info('Salt node data. Public_ip: {0}'.format(data.public_ips[0]))
         ip_address = data.public_ips[0]
 
+    if get_salt_interface(vm_) == 'private_ips':
+        salt_ip_address = data.private_ips[0]
+        log.info('Salt interface set to: {0}'.format(salt_ip_address))
+    else:
+        salt_ip_address = data.public_ips[0]
+        log.debug('Salt interface set to: {0}'.format(salt_ip_address))
+
     username = 'ec2-user'
     ssh_connect_timeout = config.get_cloud_config_value(
         'ssh_connect_timeout', vm_, __opts__, 900   # 15 minutes
     )
+    ssh_port = config.get_cloud_config_value(
+        'ssh_port', vm_, __opts__, 22
+    )
     if salt.utils.cloud.wait_for_port(ip_address, timeout=ssh_connect_timeout):
         for user in usernames:
             if salt.utils.cloud.wait_for_passwd(
@@ -418,8 +442,14 @@ def create(vm_):
                     username=user,
                     ssh_timeout=config.get_cloud_config_value(
                         'wait_for_passwd_timeout', vm_, __opts__,
-                        default=1 * 60),
-                    key_filename=key_filename):
+                        default=1 * 60
+                    ),
+                    key_filename=key_filename,
+                    known_hosts_file=config.get_cloud_config_value(
+                        'known_hosts_file', vm_, __opts__,
+                        default='/dev/null'
+                    ),
+                ):
                 username = user
                 break
         else:
@@ -431,7 +461,10 @@ def create(vm_):
     if config.get_cloud_config_value('deploy', vm_, __opts__) is True:
         deploy_script = script(vm_)
         deploy_kwargs = {
+            'opts': __opts__,
             'host': ip_address,
+            'port': ssh_port,
+            'salt_host': salt_ip_address,
             'username': username,
             'key_filename': key_filename,
             'tmp_dir': config.get_cloud_config_value(
@@ -512,7 +545,7 @@ def create(vm_):
         else:
             log.error('Failed to start Salt on Cloud VM {name}'.format(**vm_))
 
-    ret.update(data)
+    ret.update(data.__dict__)
 
     log.info('Created Cloud VM {0[name]!r}'.format(vm_))
     log.debug(
@@ -605,7 +638,9 @@ def set_tags(name, tags, call=None):
     '''
     Set tags for a node
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt-cloud -a set_tags mymachine tag1=somestuff tag2='Other stuff'
     '''
@@ -660,7 +695,9 @@ def del_tags(name, kwargs, call=None):
     '''
     Delete tags for a node
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt-cloud -a del_tags mymachine tag1,tag2,tag3
     '''
@@ -697,7 +734,9 @@ def rename(name, kwargs, call=None):
     '''
     Properly rename a node. Pass in the new name as "new name".
 
-    CLI Example::
+    CLI Example:
+
+    .. code-block:: bash
 
         salt-cloud -a rename mymachine newname=yourmachine
     '''
@@ -722,7 +761,7 @@ def rename(name, kwargs, call=None):
                 name, kwargs['newname'], exc
             ),
             # Show the traceback if the debug logging level is enabled
-            exc_info=log.isEnabledFor(logging.DEBUG)
+            exc_info_on_loglevel=logging.DEBUG
         )
     return kwargs['newname']
 
@@ -752,7 +791,7 @@ def destroy(name):
         result = libcloudfuncs_destroy(newname, get_conn())
         ret.update({'Destroyed': result})
     except Exception as exc:
-        if not exc.message.startswith('OperationNotPermitted'):
+        if not str(exc).startswith('OperationNotPermitted'):
             log.exception(exc)
             raise exc
 
@@ -761,4 +800,8 @@ def destroy(name):
                 name
             )
         )
+
+    if __opts__.get('update_cachedir', False) is True:
+        salt.utils.cloud.delete_minion_cachedir(name, __active_provider_name__.split(':')[0], __opts__)
+
     return ret
