@@ -2,9 +2,13 @@
 '''
 Manage a glusterfs pool
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import logging
+
+# Import 3rd-party libs
+from salt.ext.six.moves import range, shlex_quote as _cmd_quote  # pylint: disable=import-error,redefined-builtin
 try:
     from shlex import quote as _cmd_quote  # pylint: disable=E0611
 except ImportError:
@@ -35,6 +39,25 @@ def list_peers():
     .. code-block:: bash
 
         salt '*' glusterfs.list_peers
+
+    GLUSTER direct CLI example (to show what salt is sending to gluster):
+
+        $ gluster peer status
+
+    GLUSTER CLI 3.4.4 return example (so we know what we are parsing):
+
+        Number of Peers: 2
+
+        Hostname: ftp2
+        Port: 24007
+        Uuid: cbcb256b-e66e-4ec7-a718-21082d396c24
+        State: Peer in Cluster (Connected)
+
+        Hostname: ftp3
+        Uuid: 5ea10457-6cb2-427b-a770-7897509625e9
+        State: Peer in Cluster (Connected)
+
+
     '''
     get_peer_list = 'gluster peer status | awk \'/Hostname/ {print $2}\''
     result = __salt__['cmd.run'](get_peer_list, python_shell=True)
@@ -56,6 +79,23 @@ def peer(name):
     .. code-block:: bash
 
         salt 'one.gluster.*' glusterfs.peer two
+
+    GLUSTER direct CLI example (to show what salt is sending to gluster):
+
+        $ gluster peer probe ftp2
+
+    GLUSTER CLI 3.4.4 return example (so we know what we are parsing):
+        #if the "peer" is the local host:
+        peer probe: success: on localhost not needed
+
+        #if the peer was just added:
+        peer probe: success
+
+        #if the peer was already part of the cluster:
+        peer probe: success: host ftp2 port 24007 already in peer list
+
+
+
     '''
     if suc.check_name(name, 'a-zA-Z0-9._-'):
         return 'Invalid characters in peer name'
@@ -325,3 +365,50 @@ def delete(target, stop=True):
             return 'Volume {0} deleted'.format(target)
     else:
         return result
+
+
+def add_volume_bricks(name, bricks):
+    '''
+    Add brick(s) to an existing volume
+
+    name
+        Volume name
+
+    bricks
+        List of bricks to add to the volume
+    '''
+
+    new_bricks = []
+
+    cmd = 'echo yes | gluster volume add-brick {0}'.format(name)
+
+    if isinstance(bricks, str):
+        bricks = [bricks]
+
+    volume_bricks = status(name)
+
+    if 'does not exist' in volume_bricks:
+        return volume_bricks
+
+    if 'is not started' in volume_bricks:
+        return volume_bricks
+
+    for brick in bricks:
+        if brick in volume_bricks['bricks']:
+            log.debug('Brick {0} already in volume {1}...excluding from command'.format(brick, name))
+        else:
+            new_bricks.append(brick)
+
+    if len(new_bricks) > 0:
+        for brick in new_bricks:
+            cmd += ' '+str(brick)
+
+        result = __salt__['cmd.run'](cmd)
+
+        if result.endswith('success'):
+            return '{0} bricks successfully added to the volume {1}'.format(len(new_bricks), name)
+        else:
+            return result
+
+    else:
+        return 'Bricks already in volume {0}'.format(name)
