@@ -3,18 +3,21 @@
 Virtual machine image management tools
 '''
 
+from __future__ import absolute_import
+
 # Import python libs
 import os
 import shutil
-import yaml
 import logging
 import tempfile
 
 # Import salt libs
 import salt.crypt
 import salt.utils
+import salt.utils.cloud
 import salt.config
 import salt.syspaths
+import uuid
 
 
 # Set up logging
@@ -33,6 +36,30 @@ def _file_or_content(file_):
     return file_
 
 
+def prep_bootstrap(mpt):
+    '''
+    Update and get the random script to a random place
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' seed.prep_bootstrap /tmp
+
+    '''
+    # Verify that the boostrap script is downloaded
+    bs_ = __salt__['config.gather_bootstrap_script']()
+    fpd_ = os.path.join(mpt, 'tmp', "{0}".format(
+        uuid.uuid4()))
+    if not os.path.exists(fpd_):
+        os.makedirs(fpd_)
+    os.chmod(fpd_, 0o700)
+    fp_ = os.path.join(fpd_, os.path.basename(bs_))
+    # Copy script into tmp
+    shutil.copy(bs_, fp_)
+    return fp_
+
+
 def _mount(path, ftype):
     mpt = None
     if ftype == 'block':
@@ -142,8 +169,12 @@ def apply_(path, id_=None, config=None, approve_key=True, install=True,
     return res
 
 
-def mkconfig(config=None, tmp=None, id_=None, approve_key=True,
-            pub_key=None, priv_key=None):
+def mkconfig(config=None,
+             tmp=None,
+             id_=None,
+             approve_key=True,
+             pub_key=None,
+             priv_key=None):
     '''
     Generate keys and config and put them in a tmp directory.
 
@@ -172,7 +203,7 @@ def mkconfig(config=None, tmp=None, id_=None, approve_key=True,
     # Write the new minion's config to a tmp file
     tmp_config = os.path.join(tmp, 'minion')
     with salt.utils.fopen(tmp_config, 'w+') as fp_:
-        fp_.write(yaml.dump(config, default_flow_style=False))
+        fp_.write(salt.utils.cloud.salt_config_to_yaml(config))
 
     # Generate keys for the minion
     pubkeyfn = os.path.join(tmp, 'minion.pub')
@@ -183,8 +214,8 @@ def mkconfig(config=None, tmp=None, id_=None, approve_key=True,
             fic.write(_file_or_content(pub_key))
         with salt.utils.fopen(privkeyfn, 'w') as fic:
             fic.write(_file_or_content(priv_key))
-        os.chmod(pubkeyfn, 0600)
-        os.chmod(privkeyfn, 0600)
+        os.chmod(pubkeyfn, 0o600)
+        os.chmod(privkeyfn, 0o600)
     else:
         salt.crypt.gen_keys(tmp, 'minion', 2048)
     if approve_key and not preseeded:
@@ -203,6 +234,8 @@ def _install(mpt):
     '''
 
     _check_resolv(mpt)
+    boot_ = (prep_bootstrap(mpt)
+             or salt.syspaths.BOOTSTRAP)
     # Exec the chroot command
     cmd = 'if type salt-minion; then exit 0; '
     cmd += 'else sh {0} -c /tmp; fi'.format(salt.syspaths.BOOTSTRAP)
