@@ -1,17 +1,32 @@
 # -*- coding: utf-8 -*-
 '''
-Management of cron, the Unix command scheduler.
-===============================================
+Management of cron, the Unix command scheduler
+==============================================
 
-The cron state module allows for user crontabs to be cleanly managed.
+Cron declarations require a number of parameters. The following are the
+parameters used by Salt to define the various timing values for a cron job:
 
-Cron declarations require a number of parameters. The timing parameters need
-to be declared: minute, hour, daymonth, month, and dayweek. The user whose
-crontab is to be edited also needs to be defined.
+* ``minute``
+* ``hour``
+* ``daymonth``
+* ``month``
+* ``dayweek`` (0 to 6 are Sunday through Saturday, 7 can also be used for
+  Sunday)
 
-By default, the timing arguments are all ``*`` and the user is root. When
-making changes to an existing cron job, the name declaration is the unique
-factor, so if an existing cron that looks like this:
+.. warning::
+
+    Any timing arguments not specified take a value of ``*``. This means that
+    setting ``hour`` to ``5``, while not defining the ``minute`` param, will
+    result in Salt adding a job that will execute every minute between 5 and 6
+    A.M.!
+
+    Additionally, the default user for these states is ``root``. Therefore, if
+    the cron job is for another user, it is necessary to specify that user with
+    the ``user`` parameter.
+
+A long time ago (before 2014.2), when making changes to an existing cron job,
+the name declaration is the parameter used to uniquely identify the job,
+so if an existing cron that looks like this:
 
 .. code-block:: yaml
 
@@ -33,6 +48,48 @@ Is changed to this:
 Then the existing cron will be updated, but if the cron command is changed,
 then a new cron job will be added to the user's crontab.
 
+The current behavior is still relying on that mechanism, but you can also
+specify an identifier to identify your crontabs:
+
+.. code-block:: yaml
+
+    date > /tmp/crontest:
+      cron.present:
+        - identifier: SUPERCRON
+        - user: root
+        - minute: 7
+        - hour: 2
+
+.. versionadded:: 2014.1.2
+
+And, some months later, you modify it:
+
+.. code-block:: yaml
+
+    superscript > /tmp/crontest:
+      cron.present:
+        - identifier: SUPERCRON
+        - user: root
+        - minute: 3
+        - hour: 4
+
+.. versionadded:: 2014.1.2
+
+The old **date > /tmp/crontest** will be replaced by
+**superscript > /tmp/crontest**.
+
+Additionally, Salt also supports running a cron every ``x minutes`` very similarly to the Unix
+convention of using ``*/5`` to have a job run every five minutes. In Salt, this
+looks like:
+
+.. code-block:: yaml
+
+    date > /tmp/crontest:
+      cron.present:
+        - user: root
+        - minute: '*/5'
+
+The job will now run every 5 minutes.
 
 Additionally, the temporal parameters (minute, hour, etc.) can be randomized by
 using ``random`` instead of using a specific value. For example, by using the
@@ -58,13 +115,19 @@ from ``*`` to a randomized numeric value. However, if that field in the cron
 entry on the minion already contains a numeric value, then using the ``random``
 keyword will not modify it.
 '''
+from __future__ import absolute_import
 
 # Import python libs
 import os
+from salt.ext.six import string_types
 
 # Import salt libs
-from salt.utils import mkstemp, fopen
-from salt.modules.cron import _needs_change
+import salt.utils
+from salt.modules.cron import (
+    _needs_change,
+    _cron_matched,
+    SALT_CRON_NO_IDENTIFIER
+)
 
 
 def _check_cron(user,
@@ -73,7 +136,9 @@ def _check_cron(user,
                 hour=None,
                 daymonth=None,
                 month=None,
-                dayweek=None):
+                dayweek=None,
+                comment=None,
+                identifier=None):
     '''
     Return the changes
     '''
@@ -87,13 +152,35 @@ def _check_cron(user,
         month = str(month).lower()
     if dayweek is not None:
         dayweek = str(dayweek).lower()
+    if identifier is not None:
+        identifier = str(identifier)
+    if cmd is not None:
+        cmd = str(cmd)
     lst = __salt__['cron.list_tab'](user)
     for cron in lst['crons']:
-        if cmd == cron['cmd']:
+        if _cron_matched(cron, cmd, identifier):
             if any([_needs_change(x, y) for x, y in
                     ((cron['minute'], minute), (cron['hour'], hour),
                      (cron['daymonth'], daymonth), (cron['month'], month),
-                     (cron['dayweek'], dayweek))]):
+                     (cron['dayweek'], dayweek), (cron['identifier'], identifier),
+                     (cron['cmd'], cmd), (cron['comment'], comment))]):
+                return 'update'
+            return 'present'
+    return 'absent'
+
+
+def _check_cron_env(user,
+                    name,
+                    value=None):
+    '''
+    Return the environment changes
+    '''
+    if value is None:
+        value = ""  # Matching value set in salt.modules.cron._render_tab
+    lst = __salt__['cron.list_tab'](user)
+    for env in lst['env']:
+        if name == env['name']:
+            if value != env['value']:
                 return 'update'
             return 'present'
     return 'absent'
@@ -110,9 +197,12 @@ def _get_cron_info():
     elif __grains__['os'] == 'OpenBSD':
         group = 'crontab'
         crontab_dir = '/var/cron/tabs'
-    elif __grains__['os'] == 'Solaris':
+    elif __grains__['os_family'] == 'Solaris':
         group = 'root'
         crontab_dir = '/var/spool/cron/crontabs'
+    elif __grains__['os'] == 'MacOS':
+        group = 'wheel'
+        crontab_dir = '/usr/lib/cron/tabs'
     else:
         group = 'root'
         crontab_dir = '/var/spool/cron'
@@ -125,7 +215,9 @@ def present(name,
             hour='*',
             daymonth='*',
             month='*',
-            dayweek='*'):
+            dayweek='*',
+            comment=None,
+            identifier=None):
     '''
     Verifies that the specified cron job is present for the specified user.
     For more advanced information about what exactly can be set in the cron
@@ -137,7 +229,7 @@ def present(name,
         The command that should be executed by the cron job.
 
     user
-        The name of the user who's crontab needs to be modified, defaults to
+        The name of the user whose crontab needs to be modified, defaults to
         the root user
 
     minute
@@ -156,20 +248,31 @@ def present(name,
 
     dayweek
         The information to be set in the day of week section. Default is ``*``
+
+    comment
+        User comment to be added on line previous the cron job
+
+    identifier
+        Custom-defined identifier for tracking the cron line for future crontab
+        edits. This defaults to the state id
     '''
     name = ' '.join(name.strip().split())
+    if not identifier:
+        identifier = SALT_CRON_NO_IDENTIFIER
     ret = {'changes': {},
            'comment': '',
            'name': name,
            'result': True}
     if __opts__['test']:
         status = _check_cron(user,
-                             name,
-                             minute,
-                             hour,
-                             daymonth,
-                             month,
-                             dayweek)
+                             cmd=name,
+                             minute=minute,
+                             hour=hour,
+                             daymonth=daymonth,
+                             month=month,
+                             dayweek=dayweek,
+                             comment=comment,
+                             identifier=identifier)
         ret['result'] = None
         if status == 'absent':
             ret['comment'] = 'Cron {0} is set to be added'.format(name)
@@ -186,7 +289,9 @@ def present(name,
                                     daymonth=daymonth,
                                     month=month,
                                     dayweek=dayweek,
-                                    cmd=name)
+                                    cmd=name,
+                                    comment=comment,
+                                    identifier=identifier)
     if data == 'present':
         ret['comment'] = 'Cron {0} already present'.format(name)
         return ret
@@ -197,7 +302,7 @@ def present(name,
         return ret
 
     if data == 'updated':
-        ret['comment'] = 'Cron {0} updated'.format(name, user)
+        ret['comment'] = 'Cron {0} updated'.format(name)
         ret['changes'] = {user: name}
         return ret
     ret['comment'] = ('Cron {0} for user {1} failed to commit with error \n{2}'
@@ -208,6 +313,7 @@ def present(name,
 
 def absent(name,
            user='root',
+           identifier=None,
            **kwargs):
     '''
     Verifies that the specified cron job is absent for the specified user; only
@@ -217,14 +323,20 @@ def absent(name,
         The command that should be absent in the user crontab.
 
     user
-        The name of the user who's crontab needs to be modified, defaults to
+        The name of the user whose crontab needs to be modified, defaults to
         the root user
+
+    identifier
+        Custom-defined identifier for tracking the cron line for future crontab
+        edits. This defaults to the state id
     '''
     ### NOTE: The keyword arguments in **kwargs are ignored in this state, but
     ###       cannot be removed from the function definition, otherwise the use
     ###       of unsupported arguments will result in a traceback.
 
     name = ' '.join(name.strip().split())
+    if not identifier:
+        identifier = SALT_CRON_NO_IDENTIFIER
     ret = {'name': name,
            'result': True,
            'changes': {},
@@ -240,7 +352,7 @@ def absent(name,
             ret['comment'] = 'Cron {0} is set to be removed'.format(name)
         return ret
 
-    data = __salt__['cron.rm_job'](user, name)
+    data = __salt__['cron.rm_job'](user, name, identifier=identifier)
     if data == 'absent':
         ret['comment'] = "Cron {0} already absent".format(name)
         return ret
@@ -274,7 +386,7 @@ def file(name,
         hosted on either the salt master server, or on an HTTP or FTP server.
         For files hosted on the salt file server, if the file is located on
         the master in the directory named spam, and is called eggs, the source
-        string is salt://spam/eggs.
+        string is ``salt://spam/eggs``
 
         If the file is hosted on a HTTP or FTP server then the source_hash
         argument is also required
@@ -283,7 +395,7 @@ def file(name,
         This can be either a file which contains a source hash string for
         the source, or a source hash string. The source hash string is the
         hash algorithm followed by the hash of the file:
-        md5=e138491e9d5b97023cea823fe17bac22
+        ``md5=e138491e9d5b97023cea823fe17bac22``
 
     user
         The user to whom the crontab should be assigned. This defaults to
@@ -308,12 +420,15 @@ def file(name,
         Overrides the default backup mode for the user's crontab.
     '''
     # Initial set up
-    mode = __salt__['config.manage_mode'](600)
+    mode = __salt__['config.manage_mode']('0600')
     owner, group, crontab_dir = _get_cron_info()
 
-    cron_path = mkstemp()
-    with fopen(cron_path, 'w+') as fp_:
-        fp_.write(__salt__['cron.raw_cron'](user))
+    cron_path = salt.utils.mkstemp()
+    with salt.utils.fopen(cron_path, 'w+') as fp_:
+        raw_cron = __salt__['cron.raw_cron'](user)
+        if not raw_cron.endswith('\n'):
+            raw_cron = "{0}\n".format(raw_cron)
+        fp_.write(raw_cron)
 
     ret = {'changes': {},
            'comment': '',
@@ -321,11 +436,19 @@ def file(name,
            'result': True}
 
     # Avoid variable naming confusion in below module calls, since ID
-    # delclaration for this state will be a source URI.
+    # declaration for this state will be a source URI.
     source = name
 
-    if env is None:
-        env = kwargs.get('__env__', 'base')
+    if isinstance(env, string_types):
+        msg = (
+            'Passing a salt environment should be done using \'saltenv\' not '
+            '\'env\'. This warning will go away in Salt Boron and this '
+            'will be the default and expected behavior. Please update your '
+            'state files.'
+        )
+        salt.utils.warn_until('Boron', msg)
+        ret.setdefault('warnings', []).append(msg)
+        # No need to set __env__ = env since that's done in the state machinery
 
     if not replace and os.stat(cron_path).st_size > 0:
         ret['comment'] = 'User {0} already has a crontab. No changes ' \
@@ -341,10 +464,9 @@ def file(name,
                                              group,
                                              mode,
                                              template,
-                                             False,  # makedirs = False
                                              context,
                                              defaults,
-                                             env,
+                                             __env__,
                                              **kwargs
                                              )
         ret['result'], ret['comment'] = fcm
@@ -354,37 +476,54 @@ def file(name,
     # If the source is a list then find which file exists
     source, source_hash = __salt__['file.source_list'](source,
                                                        source_hash,
-                                                       env)
+                                                       __env__)
 
     # Gather the source file from the server
-    sfn, source_sum, comment = __salt__['file.get_managed'](cron_path,
-                                                            template,
-                                                            source,
-                                                            source_hash,
-                                                            owner,
-                                                            group,
-                                                            mode,
-                                                            env,
-                                                            context,
-                                                            defaults,
-                                                            **kwargs
-                                                            )
+    try:
+        sfn, source_sum, comment = __salt__['file.get_managed'](
+            cron_path,
+            template,
+            source,
+            source_hash,
+            owner,
+            group,
+            mode,
+            __env__,
+            context,
+            defaults,
+            **kwargs
+        )
+    except Exception as exc:
+        ret['result'] = False
+        ret['changes'] = {}
+        ret['comment'] = 'Unable to manage file: {0}'.format(exc)
+        return ret
+
     if comment:
         ret['comment'] = comment
         ret['result'] = False
         os.unlink(cron_path)
         return ret
 
-    ret = __salt__['file.manage_file'](cron_path,
-                                       sfn,
-                                       ret,
-                                       source,
-                                       source_sum,
-                                       owner,
-                                       group,
-                                       mode,
-                                       env,
-                                       backup)
+    try:
+        ret = __salt__['file.manage_file'](
+            cron_path,
+            sfn,
+            ret,
+            source,
+            source_sum,
+            owner,
+            group,
+            mode,
+            __env__,
+            backup
+        )
+    except Exception as exc:
+        ret['result'] = False
+        ret['changes'] = {}
+        ret['comment'] = 'Unable to manage file: {0}'.format(exc)
+        return ret
+
     if ret['changes']:
         ret['changes'] = {'diff': ret['changes']['diff']}
         ret['comment'] = 'Crontab for user {0} was updated'.format(user)
@@ -400,3 +539,101 @@ def file(name,
 
     os.unlink(cron_path)
     return ret
+
+
+def env_present(name,
+                value=None,
+                user='root'):
+    '''
+    Verifies that the specified environment variable is present in the crontab
+    for the specified user.
+
+    name
+        The name of the environment variable to set in the user crontab
+
+    user
+        The name of the user whose crontab needs to be modified, defaults to
+        the root user
+
+    value
+        The value to set for the given environment variable
+    '''
+    ret = {'changes': {},
+           'comment': '',
+           'name': name,
+           'result': True}
+    if __opts__['test']:
+        status = _check_cron_env(user, name, value=value)
+        ret['result'] = None
+        if status == 'absent':
+            ret['comment'] = 'Cron env {0} is set to be added'.format(name)
+        elif status == 'present':
+            ret['result'] = True
+            ret['comment'] = 'Cron env {0} already present'.format(name)
+        elif status == 'update':
+            ret['comment'] = 'Cron env {0} is set to be updated'.format(name)
+        return ret
+
+    data = __salt__['cron.set_env'](user, name, value=value)
+    if data == 'present':
+        ret['comment'] = 'Cron env {0} already present'.format(name)
+        return ret
+
+    if data == 'new':
+        ret['comment'] = 'Cron env {0} added to {1}\'s crontab'.format(name, user)
+        ret['changes'] = {user: name}
+        return ret
+
+    if data == 'updated':
+        ret['comment'] = 'Cron env {0} updated'.format(name)
+        ret['changes'] = {user: name}
+        return ret
+    ret['comment'] = ('Cron env {0} for user {1} failed to commit with error \n{2}'
+                      .format(name, user, data))
+    ret['result'] = False
+    return ret
+
+
+def env_absent(name,
+               user='root'):
+    '''
+    Verifies that the specified environment variable is absent from the crontab
+    for the specified user
+
+    name
+        The name of the environment variable to remove from the user crontab
+
+    user
+        The name of the user whose crontab needs to be modified, defaults to
+        the root user
+    '''
+
+    name = ' '.join(name.strip().split())
+    ret = {'name': name,
+           'result': True,
+           'changes': {},
+           'comment': ''}
+
+    if __opts__['test']:
+        status = _check_cron_env(user, name)
+        ret['result'] = None
+        if status == 'absent':
+            ret['result'] = True
+            ret['comment'] = 'Cron env {0} is absent'.format(name)
+        elif status == 'present' or status == 'update':
+            ret['comment'] = 'Cron env {0} is set to be removed'.format(name)
+        return ret
+
+    data = __salt__['cron.rm_env'](user, name)
+    if data == 'absent':
+        ret['comment'] = "Cron env {0} already absent".format(name)
+        return ret
+    if data == 'removed':
+        ret['comment'] = ("Cron env {0} removed from {1}'s crontab"
+                          .format(name, user))
+        ret['changes'] = {user: name}
+        return ret
+    ret['comment'] = ("Cron env {0} for user {1} failed to commit with error {2}"
+                      .format(name, user, data))
+    ret['result'] = False
+    return ret
